# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup.items import databases

import subprocess

class MariaDB(databases.Database):

    def __init__(self, conf):
        super(MariaDB, self).__init__(conf)


    def type(self):
        return 'databases/mariadb'


    def run(self):
        super(MariaDB, self).run(with_intermediate_file=True)


    def restore(self, backup_filename):
        super(MariaDB, self).restore(backup_filename,
                                     with_intermediate_file=True)


    def build_restore_command(self, backup_filename):
        command = 'mysql'

        if self.user:
            command += ' -u%s' % self.user

        if self.host:
            command += ' -h%s' % self.host

        if self.password:
            command += ' -p%s' % self.password

        return command


    def build_dump_command(self):

        command = 'mysqldump'

        if self.dump_options:
            command += ' %s' % self.dump_options

        if self.user:
            command += ' -u%s' % self.user

        if self.host:
            command += ' -h%s' % self.host

        if self.password:
            command += ' -p%s' % self.password

        if self.database == 'all':
            command += ' --all-databases'
        else:
            command += ' --databases %s' % self.database

        return command



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup.items import filesystems

import shutil

class Git(filesystems.Filesystem):


    def __init__(self, conf):
        super(Git, self).__init__(conf)
        self.branches = conf.get('branches', '--all')


    def type(self):
        return 'filesystems/git'

    
    def run(self):
        super(Git, self).run(cwd=self.path)


    def build_dump_command(self):
        if self.branches == 'all': 
            self.branches = '--all'

        command = 'git bundle create %s/%s %s' % (self.output_directory,
                                                  self.backup_file,
                                                  self.branches)
        return command

    def build_restore_command(self, backup_filename):
        try:
            shutil.rmtree(self.path)
        except OSError:
            pass
 
        command = 'git clone %s/%s -b master %s' % (self.output_directory,
                                                    backup_filename,
                                                    self.path)

        return command



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup import configuration
from swiftbackmeup import parser
from swiftbackmeup import utils
from swiftbackmeup import lists
from swiftbackmeup.items.databases import *
from swiftbackmeup.items.filesystems import *


_CONF = {
    'clean_local_copy': True,
    'create_container': True,
    'purge_backup': False,
}

_METHODS = ['list', 'list_items']

def main():

    options = parser.parse()

    if options.conf:
        _CONF['file_path'] = options.conf

    global_configuration = configuration.load_configuration(_CONF)

    configuration.verify_mandatory_parameter(global_configuration)
    backups = configuration.expand_configuration(global_configuration)
    backups = utils.filter_databases(options.items, backups)
    modes = global_configuration.get('mode')

    # swiftbackmeup restore ...
    if 'version' in options:
        for backup in backups:
            if backup['type'] == 'postgresql':
                cur_backup = postgresql.PostgreSQL(backup)
            elif backup['type'] == 'mariadb':
                cur_backup = mariadb.MariaDB(backup)
            elif backup['type'] == 'file':
                cur_backup = file.File(backup)
            elif backup['type'] == 'git':
                cur_backup = git.Git(backup)
            if options.force:
                cur_backup.restore(options.version)
            elif utils.query_yes_no('Are you sure you want to restore the backup?',
                                  default='no'):
                cur_backup.restore(options.version)
            else:
                print 'Exiting without restoring the database'

    # swiftbackmeup purge ...
    elif 'noop' in options:
        purge = False
        purged_backups = []

        if options.force:
            purge = True
        elif not options.force and utils.query_yes_no('Are you sure you want to purge the backups?',
                                                    default='no'):
            purge = True
        else:
            print 'Exiting without purging the backups'
        if purge:
            for backup in backups:
                if backup['type'] == 'postgresql':
                    cur_backup = postgresql.PostgreSQL(backup)
                elif backup['type'] == 'mariadb':
                    cur_backup = mariadb.MariaDB(backup)
                elif backup['type'] == 'file':
                    cur_backup = file.File(backup)
                elif backup['type'] == 'git':
                    cur_backup = git.Git(backup)
                purged_backups += cur_backup.purge(modes[options.mode], options.noop)
            lists.list_purged_backups(purged_backups, options.noop)

    # swiftbackmeup backup ...
    else:
        # If a --list-* options has been specified, call the proper method
        # and exit
        #
        for method in _METHODS:
            if getattr(options, method):
                if method == 'list_items':
                    lists.list_items(backups, options)
                elif method == 'list':
                    lists.list_remote_backups(backups, options, modes)
                return


        for backup in backups:
            if options.mode in backup['subscriptions']:
                backup['filename'] = utils.build_filename(backup,
                                                          modes[options.mode])
                if backup['type'] == 'postgresql':
                    cur_backup = postgresql.PostgreSQL(backup)
                elif backup['type'] == 'mariadb':
                    cur_backup = mariadb.MariaDB(backup)
                elif backup['type'] == 'file':
                    cur_backup = file.File(backup)
                elif backup['type'] == 'git':
                    cur_backup = git.Git(backup)
                cur_backup.run()
                cur_backup.upload()



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup import exceptions

import os
import yaml

_FIELDS = ['os_username', 'os_password', 'os_tenant_name', 'os_auth_url',
           'create_container', 'purge_container', 'swift_container',
           'swift_pseudo_folder', 'output_directory', 'clean_local_copy',
           'type', 'user', 'password', 'host', 'port',
           'store_type']


def check_configuration_file_existence(configuration_file_path=None):
    """Check if the configuration file is present."""

    if configuration_file_path:
        if not os.path.exists(configuration_file_path):
            raise exceptions.ConfigurationExceptions('File %s does not exist' % configuration_file_path)
        file_path = configuration_file_path
    elif os.getenv('SWIFTBACKMEUP_CONFIGURATION'):
        if not os.path.exists(os.getenv('SWIFTBACKMEUP_CONFIGURATION')):
            raise exceptions.ConfigurationExceptions('File %s does not exist' % os.getenv('SWIFTBACKMEUP_CONFIGURATION'))
        file_path = os.getenv('SWIFTBACKMEUP_CONFIGURATION')
    else:
        if not os.path.exists('/etc/swiftbackmeup.conf'):
            raise exceptions.ConfigurationExceptions(
                'File /etc/swiftbackmeup.conf does not exist (you could specify an alternate location using --conf)'
            )
        file_path = '/etc/swiftbackmeup.conf'

    return file_path


def load_configuration(conf):
    """Load the swiftbackmeup configuration file."""

    file_path = check_configuration_file_existence(conf.get('file_path'))

    try:
        file_path_content = open(file_path, 'r').read()
    except IOError as exc:
        raise exceptions.ConfigurationExceptions(exc)

    try:
        conf = yaml.load(file_path_content)
    except yaml.YAMLError as exc:
        raise exceptions.ConfigurationExceptions(exc)

    return conf


def expand_configuration(configuration):
    """Fill up backups with defaults."""

    for backup in configuration['backups']:
        for field in _FIELDS:
            if field not in backup or backup[field] is None:
                if field not in configuration:
                    backup[field] = None
                else:
                    backup[field] = configuration[field]

    return configuration['backups']


def verify_mandatory_parameter(configuration):
    """Ensure that all mandatory parameters are in the configuration object."""

    # Swift Parameters
    os_username = configuration.get('os_username', os.getenv('OS_USERNAME'))
    os_password = configuration.get('os_password',os.getenv('OS_PASSWORD'))
    os_tenant_name = configuration.get('os_tenant_name', os.getenv('OS_TENANT_NAME'))
    os_auth_url = configuration.get('os_auth_url', os.getenv('OS_AUTH_URL'))

    if not (os_username and os_password and os_tenant_name and os_auth_url):
        raise exceptions.ConfigurationExceptions('One of the following parameter is not configured: os_username, os_password, os_tenant_name, os_auth_url')

    if (len([1 for backup in configuration['backups'] if 'swift_container' in backup]) != len(configuration['backups'])) and 'swift_container' not in configuration:
        raise exceptions.ConfigurationExceptions('swift_container has not been specified for every backups and no global setting has been set')


    # Backup Parameters
    if 'backups' not in configuration:
        raise exceptions.ConfigurationExceptions('No backups field encountered')

    if len(configuration['backups']) == 0:
        raise exceptions.ConfigurationExceptions('Backups has no backup configured')

    for backup in configuration['backups']:
        if 'name' not in backup:
            raise exceptions.ConfigurationExceptions('A backup has the name field missing')



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup.stores import swift

import datetime
import exceptions
import os
import subprocess


class Item(object):


    def __init__(self, conf):
        self.name = conf.get('name')
        self.env = os.environ

        self.backup_file = conf.get('filename')
        self.backup_filename = conf.get('backup_filename')
        self.backup_filename_prefix = conf.get('backup_filename_prefix')
        self.backup_filename_suffix = conf.get('backup_filename_suffix')
        self.output_directory = conf.get('output_directory')
        self.clean_local_copy = conf.get('clean_local_copy')

        self.store_type = conf.get('store_type')
        self.store = self.get_store(conf)

        self.create_container = conf.get('create_container')
        self.purge_container = conf.get('purge_container')

        # store_type: swift
        self.swift_container = conf.get('swift_container')
        self.swift_pseudo_folder = conf.get('swift_pseudo_folder')


    def get_store(self, conf):
        """Method to retrieve a connection to the remote store."""

        if self.store_type == 'swift':
            store = swift.Swift(conf)
        else:
            raise Exception('Unknown store_type: %s' % self.store_type)

        return store


    def type(self):
        """Method to retrieve the type object the backup was made from."""
        pass


    def run(self, with_intermediate_file=False, cwd=None):
        """Method to run the backup command where it applies."""

        command = self.build_dump_command()

        if with_intermediate_file:
            try:
                backup_file_f = open('%s/%s' % (self.output_directory,
                                                self.backup_file), 'w')
            except IOError as exc:
                raise

            p = subprocess.Popen(command.split(), stdout=backup_file_f,
                                 env=self.env, cwd=cwd)
            p.wait()
            backup_file_f.flush()
        else:
            FNULL = open(os.devnull, 'w')
            p = subprocess.Popen(command.split(), env=self.env, cwd=cwd,
                                 stdout=FNULL, stderr=subprocess.STDOUT)


    def restore(self, backup_filename,with_intermediate_file=False):
        """Method to restore the backup."""

        self.store.get(self.swift_container, backup_filename,
                       self.output_directory)
        command = self.build_restore_command(backup_filename)


        if with_intermediate_file:
            file_path = '%s/%s' % (self.output_directory, backup_filename)
            backup_file_content = open(file_path, 'r').read()

            p = subprocess.Popen(command.split(), stdin=subprocess.PIPE)
            p.communicate(backup_file_content)
        else:
            FNULL = open(os.devnull, 'w')
            p = subprocess.Popen(command.split(), stdout=FNULL,
                                 stderr=subprocess.STDOUT)
            p.wait()

        if self.clean_local_copy:
            self._clean_local_copy(backup_filename)


    def purge(self, mode, noop=False):
        """Method to purge the remote backups."""

        backups = self.store.list(self.name, self.type(),
                                  self.swift_container,
                                  self.backup_filename,
                                  self.swift_pseudo_folder,
                                  self.backup_filename_prefix,
                                  self.backup_filename_suffix)

        if mode['unit'] == 'item':
            if len(backups) > mode['retention']:
                backups = backups[:-mode['retention']]
            else:
                backups = []
        else:
            tmp_backup = []
            for backup in backups:
                if (datetime.datetime.now() - datetime.datetime.strptime(backup['last-modified'], '%Y-%m-%dT%H:%M:%S.%f')).days >= mode['retention']:
                    tmp_backup.append(backup)
            backups = tmp_backup

        if not noop:
            for backup in backups:
                self.store.delete(self.swift_container, backup['filename'])

        return backups


    def list(self):
        """Method to list the backups of a given item on the remote store."""

        return self.store.list(self.name, self.type(), self.swift_container,
                               self.backup_filename, self.swift_pseudo_folder,
                               self.backup_filename_prefix,
                               self.backup_filename_suffix)


    def upload(self):
        """Method to upload a backup of a given item on the remote store."""

        self.store.upload(self.swift_container,
                          '%s/%s' % (self.output_directory, self.backup_file),
                          self.swift_pseudo_folder, self.create_container)
        if self.clean_local_copy:
            self._clean_local_copy(self.backup_file)


    def build_restore_command(self, backup_filename):
        """Method to build the restore command that will be run."""
        pass


    def build_dump_command(self):
        """Method to build the dump command that will be run."""
        pass


    def _clean_local_copy(self, backup_file=None):

        if not backup_file:
            self.backup_file

        try:
            os.remove('%s/%s' % (self.output_directory, backup_file))
        except OSError:
            raise



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prettytable import PrettyTable

import datetime
import subprocess
import sys

def build_filename(backup, mode):

    if 'backup_filename' in backup:
        backup_path = backup['backup_filename']
    else:
        backup_path = backup.get('backup_filename_prefix', backup['name'])
        backup_path += datetime.datetime.now().strftime(mode['pattern'])
        if 'backup_filename_suffix' in backup:
            backup_path += '%s' % backup['backup_filename_suffix']

    return backup_path


def output_informations(data):

    x = PrettyTable()
    for column in data:
        x.add_column(column[0], column[1])
    print x


def filter_databases(databases, backups):
    if isinstance(databases, list):
        tmp_backups = []
        for backup in backups:
            if backup['name'] in databases:
                tmp_backups.append(backup)
        backups = tmp_backups
    return backups


def get_file_type(path):
    p = subprocess.Popen(['file', path], stdout=subprocess.PIPE)
    return p.communicate()[0]


def query_yes_no(question, default="yes"):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
        It must be "yes" (the default), "no" or None (meaning
        an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = raw_input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup.items import filesystems


class File(filesystems.Filesystem):


    def __init__(self, conf):
        super(File, self).__init__(conf)


    def type(self):
        return 'filesystems/file'


    def run(self):
        super(File, self).run(with_intermediate_file=True)


    def build_dump_command(self):
        return 'cat %s' % self.path


    def build_restore_command(self, backup_filename):
        return 'cp -p %s/%s %s' % (self.output_directory,
                                   backup_filename,
                                   self.path)



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse

def parse():

    parser = argparse.ArgumentParser(description='swiftbackmeup')

    parser.add_argument('--conf',
        help='Path to configuration file')


    subparsers = parser.add_subparsers(help='commands')

    backup_parser = subparsers.add_parser('backup', help='backup configured items')
    backup_parser.add_argument('--mode',
        default='now',
        help='Mode under which the script will be run')
    backup_parser.add_argument('--list',
        action='store_true',
        help='List all remote backups')
    backup_parser.add_argument('--list-items',
        action='store_true',
        help='List all currently configured backups')
    backup_parser.add_argument('--items',
        action='append',
        nargs='*',
        help='Items list to backup')

    purge_parser = subparsers.add_parser('purge', help='purge stored backups')
    purge_parser.add_argument('--noop',
        action='store_true',
        help='List the backups to be purged, but don\'t purge them')
    purge_parser.add_argument('--force',
        action='store_true',
        help='Force answer yes to security question')
    purge_parser.add_argument('--items',
        action='append',
        nargs='*',
        help='Items backups to purge')
    purge_parser.add_argument('--mode',
        default='now',
        help='Mode under which the script will be run')

    restore_parser = subparsers.add_parser('restore', help='restore item from remote backups')
    restore_parser.add_argument('--version',
        default='latest',
        help='name of the backup to restore')
    restore_parser.add_argument('--items',
        action='append',
        nargs='*',
        help='Items to restore')
    restore_parser.add_argument('--force',
        action='store_true',
        help='Force answer yes to security question')

    options = parser.parse_args()
    normalize_items_parameter(options)

    return options


def normalize_items_parameter(options):
    """The items parameters can have differents form based on how it
       was passed as an input

       swiftbackmeup --items db1,db2
       swiftbackmeup --items db1 db2
       swiftbackmeup --items db1 --items db2

       This method aims to provide a plain array witch each element being
       a items itself
    """

    if not isinstance(options.items, list):
        return

    final_dbs = []
    for dbs in options.items:
        for db in dbs:
            if ',' in db:
                final_dbs += db.split(',')
            else:
                final_dbs.append(db)

    options.items = final_dbs



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from prettytable import PrettyTable
from swiftbackmeup import utils
from swiftbackmeup.items.databases import postgresql, mariadb
from swiftbackmeup.items.filesystems import file, git

_FULL_TYPE = {
    'mariadb': 'databases/mariadb',
    'postgresql': 'databases/postgres',
    'file': 'filesystem/file',
    'git': 'filesystem/git',
}

# If --list-items has been specified, list the backups items configured
# in the configuration file
#
def list_items(backups, options):
    result = [['Item', []],
              ['Type', []],
              ['Host', []],
              ['Swift Container', []],
              ['Swift Pseudo-Folder', []],
              ['Subscriptions', []]]
    for backup in backups:
        if options.mode in backup['subscriptions']:
            result[0][1].append(backup['name'])
            result[1][1].append(_FULL_TYPE[backup['type']])
            result[2][1].append(backup['host'])
            result[3][1].append(backup['swift_container'])
            result[4][1].append(backup['swift_pseudo_folder'])
            result[5][1].append(', '.join(backup['subscriptions']))
    utils.output_informations(result)


# If --list has been specified, list the backups available
# on the specified store
#
def list_remote_backups(backups, options, modes):
    result = [['Item', []],
              ['Type', []],
              ['Backup file', []],
              ['Last Modified', []]]
    for backup in backups:
        if options.mode in backup['subscriptions']:
            backup['filename'] = utils.build_filename(backup,
                                                      modes[options.mode])
            if backup['type'] == 'postgresql':
                cur_backup = postgresql.PostgreSQL(backup)
            elif backup['type'] == 'mariadb':
                cur_backup = mariadb.MariaDB(backup)
            elif backup['type'] == 'file':
                cur_backup = file.File(backup)
            elif backup['type'] == 'git':
                cur_backup = git.Git(backup)
            
            for backup_item in cur_backup.list():
                result[0][1].append(backup_item['item'])
                result[1][1].append(backup_item['type'])
                result[2][1].append(backup_item['filename'])
                result[3][1].append(backup_item['last-modified'])
    utils.output_informations(result)


def list_purged_backups(backups, noop):
    result = [['Item', []],
              ['Backup file', []],
              ['Last Modified', []],
              ['Status', []]]

    status = 'Purged'
    if noop:
        status += ' (noop)'

    for backup in backups:
        result[0][1].append(backup['item'])
        result[1][1].append(backup['filename'])
        result[2][1].append(backup['last-modified'])
        result[3][1].append(status)
    utils.output_informations(result)



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup import exceptions
from swiftbackmeup import utils
from swiftbackmeup.items import databases


_PARAMS = {
  'data_only': '-a',
  'globals_only': '-g',
  'roles_only': '-r',
  'schema_only': '-s',
  'tablespaces_only': '-t'
}

class PostgreSQL(databases.Database):

    def __init__(self, conf):
        super(PostgreSQL, self).__init__(conf)
        self.data_only = conf.get('data_only')
        self.globals_only = conf.get('globals_only')
        self.roles_only = conf.get('roles_only')
        self.schema_only = conf.get('schema_only')
        self.tablespaces_only = conf.get('tablespaces_only')


    def type(self):
        return 'databases/postgresql'


    def run(self):
        super(PostgreSQL, self).run(with_intermediate_file=True)


    def build_restore_command(self, backup_filename):
        file_path = '%s/%s' % (self.output_directory, backup_filename)
        file_type = utils.get_file_type(file_path)

        if 'ASCII text' in file_type:
            command = 'psql -f %s' % file_path
        elif 'PostgreSQL custom database dump' in file_type:
            command = 'pg_restore --clean -d %s' % self.database
        else:
            raise exceptions.DatabaseExceptions('%s: Not a supported file type for PostgreSQL backups' % file_type)

        if self.user:
            command += ' -U %s' % self.user

        if self.host:
            command += ' -h %s' % self.host

        if self.password:
            self.env['PGPASSWORD'] = self.password

        if 'psql' in command:
            command += ' %s' % self.database

        if 'pg_restore' in command:
            command += ' %s' % file_path

        return command


    def build_dump_command(self):

        if self.database == 'all':
            command = 'pg_dumpall'
        else:
            command = 'pg_dump'

        # pg_dumpall *-only options management
        if self.globals_only and self.roles_only:
            raise exceptions.ConfigurationExceptions('%s: options globals_only and roles_only cannot be used together' % self.database)
        elif self.globals_only and self.tablespaces_only:
            raise  exceptions.ConfigurationExceptions('%s: options globals_only and tablespaces_only cannot be used together' % self.database)
        elif self.tablespaces_only and self.roles_only:
            raise  exceptions.ConfigurationExceptions('%s: options tablespaces_only and roles_only cannot be used together' % self.database)

        for param in _PARAMS.keys():
            if getattr(self, param, None):
                command += ' %s' %  _PARAMS[param]

        if self.dump_options and not self.database == 'all':
            command += ' %s' % self.dump_options

        if self.user:
            command += ' -U %s' % self.user

        if self.host:
            command += ' -h %s' % self.host

        if self.database and not self.database == 'all':
            command += ' %s' % self.database

        if self.password:
            self.env['PGPASSWORD'] = self.password

        return command



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup import items


class Filesystem(items.Item):


    def __init__(self, conf):
        super(Filesystem, self).__init__(conf)
        self.path = conf.get('path')



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import codecs
import os
import setuptools

from swiftbackmeup import version


def _get_requirements():
    requirements_path = '%s/%s' % (os.path.dirname(os.path.abspath(__file__)),
                                   'requirements.txt')
    with open(requirements_path, 'r') as f:
        requirements = f.read()
        # remove the dependencies which comes from url source because
        # it's not supported by install_requires
        return [dep for dep in requirements.split('\n')
                if not dep.startswith('-e')]


def _get_readme():
    readme_path = '%s/%s' % (os.path.dirname(os.path.abspath(__file__)),
                             'README.md')

    with codecs.open(readme_path, 'r', encoding='utf8') as f:
        return f.read()


setuptools.setup(
    name='swiftbackmeup',
    version=version.__version__,
    packages=setuptools.find_packages(),
    author='Yanis Guenane',
    author_email='yguenane@redhat.com',
    description='Tools to backup databases to OpenStack Object Store Swift',
    long_description=_get_readme(),
    install_requires=_get_requirements(),
    url='https://github.com/redhat-cip/swiftbackmeup',
    license='Apache v2.0',
    include_package_data=True,
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ],
    entry_points={
        'console_scripts': [
            'swiftbackmeup = swiftbackmeup.shell:main'
        ],
    }
)



# Copyright 2016 Yanis Guenane <yguenane@redhat.com>
# Author: Yanis Guenane <yguenane@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from swiftbackmeup import stores
from swiftbackmeup import exceptions

import os
import re
import swiftclient

class Swift(stores.Store):

    def __init__(self, conf):
        self.os_username = conf.get('os_username',
                                    os.getenv('OS_USERNAME'))
        self.os_password = conf.get('os_password',
                                    os.getenv('OS_USERNAME'))
        self.os_tenant_name = conf.get('os_tenant_name',
                                       os.getenv('OS_TENANT_NAME'))
        self.os_auth_url = conf.get('os_auth_url',
                                    os.getenv('OS_AUTH_URL'))
        self.connection = self.get_connection()


    def get_connection(self):
        return swiftclient.client.Connection(auth_version='2',
                                             user=self.os_username,
                                             key=self.os_password,
                                             tenant_name=self.os_tenant_name,
                                             authurl=self.os_auth_url)


    def delete(self, container, filename):
        try:
            self.connection.delete_object(container, filename)
        except swiftclient.exceptions.ClientException as exc:
            raise exceptions.StoreExceptions('An error occured while deleting %s' % filename)


    def get(self, container, filename, output_directory):
        try:
            resp_headers, obj_contents = self.connection.get_object(container,
                                                                    filename)
        except swiftclient.exceptions.ClientException as exc:
            if exc.http_reason == 'Not Found':
                raise exceptions.StoreExceptions('%s: File not found in store' % filename)

        backup_directory = os.path.dirname('%s/%s' % (output_directory, filename))
        if not os.path.exists(backup_directory):
            os.makedirs(backup_directory)

        with open('%s/%s' % (output_directory, filename), 'w') as backup:
            backup.write(obj_contents)


    def list(self, item, item_type, container, filename=None, pseudo_folder=None,
             filename_prefix=None, filename_suffix=None):

        if pseudo_folder:
            if filename:
                backup_name_pattern = '%s/%s' % (pseudo_folder,
                                                 filename)
            else:
                backup_name_pattern = pseudo_folder or ''
                if filename_prefix and filename_suffix:
                    backup_name_pattern += '/%s.*%s' % (filename_prefix,
                                                        filename_suffix)
                elif filename_prefix and not filename_suffix:
                    backup_name_pattern += '/%s.*' % filename_prefix
                elif not filename_prefix and filename_suffix:
                    backup_name_pattern += '/.*%s' % filename_suffix

        else:
            if filename:
                backup_name_pattern = filename
            else:
                backup_name_pattern = ''
                if filename_prefix and filename_suffix:
                    backup_name_pattern += '%s.*%s' % (filename_prefix,
                                                        filename_suffix)
                elif filename_prefix and not filename_suffix:
                    backup_name_pattern += '%s.*' % filename_prefix
                elif not filename_prefix and filename_suffix:
                    backup_name_pattern += '.*%s' % filename_suffix

        resp, data = self.connection.get_container(container)

        result = []
        for backup in data:
            m = re.search(backup_name_pattern, backup['name'])
            if m:
                result.append({'item': item,
                               'type': item_type,
                               'filename': m.group(0),
                               'last-modified': backup['last_modified']})
        return result


    def upload(self, container, file_path, pseudo_folder=None,
               create_container=True):
        try:
            self.connection.head_container(container)
        except swiftclient.exceptions.ClientException as exc:
            if exc.http_reason == 'Not Found' and create_container:
                self.connection.put_container(container)

        backup_file_content = open(file_path, 'r').read()

        if pseudo_folder:
            swift_path = '%s/%s' % (pseudo_folder, os.path.basename(file_path))
        else:
            swift_path = os.path.basename(file_path)

        self.connection.put_object(container, swift_path,
                                   backup_file_content)
